<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: text2html.php 81 2008-08-08 13:42:03Z lks $
 */

/** zfilter */
require_once 'zengine/classes/zfilter.php';

/**
 * Реализует интерфейс фильтра. Помещает абзацы в параграфы, формирует
 * автоматические списки (нумерованные, ненумерованные) с возможной вложенностью
 *
 * @package    zengine
 * @subpackage zfilter
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zfilter_text2html implements zfilter {
    /**
     * Функция обрабатывает текст и возвращает HTML-код. Происходит удаление всех
     * возвратов каретки (\r) со всего текста.
     *
     * @param string $text
     * @return string
     */
    public function filter ($text)
    {
        // Убираем все переводы каретки (оставляем только перенос на новую строку)
        $text = str_replace("\r", "", $text);
        // Если в конце текста нет перехода на новую строку, то добавляем его
        if (! preg_match("/\n$/", $text)) {
            $text .= "\n";
        }
        // Обрамляем в параграфы все, что есть (пропускаем списки)
        $text = preg_replace("/^([^\n#-][^\n]*)\n/m", "<p>$1</p>\n", $text);
        // Заголовки не обрамляются в параграфы
        $text = preg_replace("/^<p>(<h[1-6]?>.+<\/h[1-6]?>)<\/p>$/im", "$1", $text);
        // Если между двумя параграфами нету пустой строки, то используем <br />
        $text = preg_replace("/<\/p>\n<p>/", "<br />\n", $text);
        // Обрабатываем списки
        $text = $this->parseLists($text);
        $text = preg_replace('%(</?(?:ul|li|ol)>)<br />%', '$1', $text);
        $text = preg_replace('%<p>(<(?:o|u)l>)%', '$1', $text);
        $text = preg_replace('%(</(?:o|u)l>)</p>%', '$1', $text);
        return $text;
    }

    /**
     * Функция формирует HTML-списки из маркеров
     *
     * @param string $text текст, который нужно обработать
     * @return string
     */
    private function parseLists ($text)
    {
        // Обработка списков
        // Неупорядоченный список
        if (preg_match("/^-/m", $text)) {
            $text = preg_replace("/^([#-]\s*.+)$/m", "[li]$1[/li]", $text);
            $text = preg_replace("/\[li\].+\[\/li\]/", "[ul]\n$0\n[/ul]", $text);
            $text = preg_replace("/\n\[\/ul\]\n\[ul\]/", "", $text);
            // Обрабатываем вложенные списки
            preg_match_all("/\[ul\]\n(.+)\n\[\/ul\]/Us", $text, $matches);
            foreach ($matches[1] as $matchKey => $ul) {
                // Массив, в котором будет сформирован список после обработки
                $arLists = array();
                $textLines = explode("\n", $ul);
                $levelItem = array(&$arLists);
                $prevLevel = 0;
                $i = 0;
                foreach ($textLines as $line) {
                    $i ++;
                    $line = substr($line, 4, strlen($line) - 9);
                    $level = strspn($line, '-#') - 1;
                    // Увеличиваем отступ
                    if ($level > $prevLevel) {
                        $levelItem[$level] = &$prevItem;
                    }
                    // Определяем тип списка
                    if ($line{$level} == '-') {
                        $type = 'ul';
                    } else {
                        $type = 'ol';
                    }
                    // Формируем информацию об элементе
                    $line = ltrim($line, ' -#');
                    $item[$i] = array();
                    $item[$i]['text'] = $line;
                    // Записываем элемент
                    $levelItem[$level][$type][] = &$item[$i];
                    $prevItem = &$item[$i];
                    $prevLevel = $level;
                }
                $text = str_replace($matches[0][$matchKey], $this->_processList($arLists), $text);
            }
        }
        return $text;
    }

    /**
     * Функция строит HTML-список в соответствии со структурой массива
     *
     * @param array $array
     * @return string
     */
    private function _processList ($array)
    {
        $result = '';
        foreach ($array as $key => $items) {
            if ($key != 'ul' && $key != 'ol') {
                continue;
            }
            $result .= "<{$key}>\n";
            foreach ($items as $item) {
                $result .= "<li>{$item['text']}";
                if (isset($item['ul']) || isset($item['ol'])) {
                    $result .= "\n" . $this->_processList($item);
                }
                $result .= "</li>\n";
            }
            $result .= "</{$key}>";
        }
        return $result;
    }
}
