<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zpagesorter.php 491 2008-11-11 14:13:37Z lks $
 */


/**
 * Класс для сортировки страниц
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zpagesorter {
    /**
     * Конструктор
     *
     * @param string $orders правила сортировки
     */
    public function __construct ($orders) {
        $this->setSortOrders($orders);
    }


    /**
     * Сортирует страницы $pages в соответствии с указанными правилами
     *
     * @param array $pages
     * @param string $orders
     */
    public function sort (&$pages) {
        usort($pages, array($this, 'cmp'));
    }

    /**
     * Возвращает правила сортировки пунктов меню для subpages
     *
     * @param array $orders
     * @return array
     */
    private function setSortOrders ($orders) {
        $this->_sortOrders = array();
        foreach (preg_split("%,\s+%", strtolower(trim($orders))) as $order) {
            $parts = explode(' ', $order, 2);
            if (empty($parts[1])) {
                $parts[1] = 'asc';
            }
            $this->_sortOrders[$parts[0]] = $parts[1];
        }
        if (empty($this->_sortOrders)) {
            $this->_sortOrders = array('title' => 'asc');
        }
    }

    /**
     * Возвращает массив правил сортировки
     *
     * @return array
     */
    public function getSortOrders () {
    	return $this->_sortOrders;
    }

    /**
     * Метод сортировки страниц между собой.
     *
     * @param zpage|zmenu $left
     * @param zpage|zmenu $right
     * @return int
     */
    public function cmp ($left, $right) {
        foreach ($this->_sortOrders as $order => $by) {
            $leftPart = $rightPart = '';

            if (method_exists($left, 'get' . $order)) {
                $leftPart = $left->{'get' . $order}();
                $rightPart = $right->{'get' . $order}();
            } else {
                $leftPart = $left->getProperty($order);
                $rightPart = $right->getProperty($order);
            }

            $fnCmp = 'strcmp';
            if ('sort' == $order) {
                $fnCmp = 'intcmp';
            }

            switch ($by) {
                case 'desc':
                    $cmp = $this->$fnCmp($rightPart, $leftPart);
                    break;
                case 'asc':
                default:
                    $cmp = $this->$fnCmp($leftPart, $rightPart);
            }

            if ($cmp != 0) {
                return $cmp;
            }
        }

        return 0;
    }

    /**
     * Функция сравнения чисел
     *
     * @param integer $left
     * @param integer $right
     * @return integer
     */
    public function intcmp ($left, $right) {
    	if ((int) $left == (int) $right) {
    	    return 0;
    	}

    	return (int) $left < (int) $right ? -1 : 1;
    }

    /**
     * Прокси для функции сравнения строк
     *
     * @param string $left
     * @param string $right
     * @return integer
     */
    public function strcmp ($left, $right) {
    	return strcmp($left, $right);
    }

    /**
     * Массив с информацией о правилах сортировки страниц
     *
     * @var array
     */
    private $_sortOrders;
}
