<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zsource.php 546 2008-12-18 08:10:45Z lks $
 */


/**
 * Интерфейс источников данных
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
interface zsource {
    /**
     * Удаляет страницу по пути $pagepath и все её дочерние страницы
     *
     * @param string $pagepath
     * @return boolean
     */
    public function delete ($pagepath);

    /**
     * Возвращает одномерный ассоциативный массив со всеми страницами, хранящимися в
     * папке content. Ключ и значение массива - путь к странице.
     *
     * @param string $parent путь к корневой странице, от которой будет осуществляться поиск
     * @return array
     */
    public function getAllPages ($parent = '');

    /**
     * Возвращает массив дочерних страниц
     *
     * @param string $pagepath путь к родительской странице
     * @return array
     */
    public function getChildren ($pagepath);

    /**
     * Метод должен вернуть объект zmenu с заполненной структурой меню
     *
     * @return zmenu
     */
    public function getMenu ();

    /**
     * Возвращает true, если страница существует, в противном случае - false
     *
     * @return boolean
     */
    public function exists ($pagepath);

    /**
     * Возвращает свойства запрошенной страницы
     *
     * Обязательные свойства страницы:
     *      title - название страницы
     *      body - текст страницы
     *
     * Необязательные свойства страницы
     *      keywords - ключевые слова страницы
     *      description - описание страницы
     *      heading - заголовк страницы
     *      menuitem - название пункта меню
     *      template - название шаблона
     *
     * @param string $pagepath путь к странице
     * @return array ассоциативный массив свойств страницы
     */
    public function load ($pagepath);

    /**
     * Возвращает время последней правки файла страницы
     *
     * @param string $pagepath путь к странице
     * @return int
     */
    public function mtime ($pagepath);

    /**
     * Возвращает свойства из строки
     *
     * @param string $content
     * @return array
     */
    public function parse ($content);

    /**
     * Сохраняняет информацию о странице в путь $pagename.
     *
     * @param string $pagename путь к странице
     * @param array $properties ассоциативный массив свойств страницы
     * @return boolean
     */
    public function save ($pagename, array $properties);

    /**
     * Начинает транзакцию
     *
     */
    public function beginTransaction ();

    /**
     * Подтверждает изменения, внесенные во время транзакции
     *
     */
    public function commit ();

    /**
     * Откатывает изменения, внесенные во время транзакции
     *
     */
    public function rollBack ();

    /**
     * Сбрасывает кэш и обновляет необходимую информацию
     *
     */
    public function update ();
}
